# This file is MACHINE GENERATED! Do not edit.
# Generated by: tensorflow/python/tools/api/generator/create_python_api.py script.
"""Testing.

See the [Testing](https://tensorflow.org/api_docs/python/tf/test) guide.

Note: `tf.compat.v1.test.mock` is an alias to the python `mock` or
`unittest.mock` depending on the python version.

"""

from __future__ import print_function as _print_function

import sys as _sys

from tensorflow.python.framework.test_util import TensorFlowTestCase as TestCase
from tensorflow.python.framework.test_util import assert_equal_graph_def_v1 as assert_equal_graph_def
from tensorflow.python.framework.test_util import create_local_cluster
from tensorflow.python.framework.test_util import gpu_device_name
from tensorflow.python.framework.test_util import is_gpu_available
from tensorflow.python.ops.gradient_checker import compute_gradient
from tensorflow.python.ops.gradient_checker import compute_gradient_error
from tensorflow.python.platform.benchmark import TensorFlowBenchmark as Benchmark
from tensorflow.python.platform.benchmark import benchmark_config
from tensorflow.python.platform.googletest import StubOutForTesting
from tensorflow.python.platform.googletest import mock
from tensorflow.python.platform.test import get_temp_dir
from tensorflow.python.platform.test import is_built_with_cuda
from tensorflow.python.platform.test import is_built_with_gpu_support
from tensorflow.python.platform.test import is_built_with_rocm
from tensorflow.python.platform.test import main
from tensorflow.python.platform.test import test_src_dir_path

del _print_function

from tensorflow.python.util import module_wrapper as _module_wrapper

if not isinstance(_sys.modules[__name__], _module_wrapper.TFModuleWrapper):
  _sys.modules[__name__] = _module_wrapper.TFModuleWrapper(
      _sys.modules[__name__], "compat.v1.test", public_apis=None, deprecation=False,
      has_lite=False)
