/*******************************************************************************
* Copyright 2019 Intel Corporation
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*     http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*******************************************************************************/

#ifndef MKLDNN_VERSION_H
#define MKLDNN_VERSION_H

/* Major version of MKL-DNN */
#define MKLDNN_VERSION_MAJOR 0

/* Minor version of MKL-DNN */
#define MKLDNN_VERSION_MINOR 20

/* Patch version of MKL-DNN */
#define MKLDNN_VERSION_PATCH 3

/* Git Commit Hash of MKL-DNN */
#define MKLDNN_VERSION_HASH  "N/A"

#endif
